@@ -19,6 +19,7 @@
 
 
 
+#include <cstdlib>
 #include <numeric>
 #include <complex>
 #include <pyublas/python_helpers.hpp>
@@ -34,14 +35,23 @@
 
 
 
+namespace
+{
+  static struct _pyublas_array_importer
+  {
+    _pyublas_array_importer()
+    { 
+      import_array();
+    }
+  } _array_importer;
+}
+
+
+
+
+
 namespace pyublas
 {
-  static struct array_importer
-  {
-    array_importer()
-    { import_array(); }
-  } _array_importer;
-
   inline NPY_TYPES get_typenum(bool) { return NPY_BOOL; }
   inline NPY_TYPES get_typenum(npy_bool) { return NPY_BOOL; }
   inline NPY_TYPES get_typenum(npy_byte) { return NPY_BYTE; }
@@ -139,6 +149,14 @@ namespace pyublas
       numpy_array(const boost::python::handle<> &obj)
         : m_numpy_array(obj)
       {
+        if (!obj.get())
+          return;
+        if (obj.get() == Py_None)
+        {
+          m_numpy_array = boost::python::handle<>();
+          return;
+        }
+
         if (!PyArray_Check(obj.get()))
           PYUBLAS_PYERROR(TypeError, "argument is not a numpy array");
         if (PyArray_TYPE(obj.get()) != get_typenum(T()))
@@ -193,8 +211,13 @@ namespace pyublas
 
       size_type size() const 
       { 
+        if (!is_valid())
+          return 0;
+
         if (ndim() != 0)
-          return min_stride()/sizeof(T)*size_from_dims(ndim(), dims());
+        {
+          return end()-begin();
+        }
         else
           return 1;
       }
@@ -206,15 +229,13 @@ namespace pyublas
       { return PyArray_NDIM(m_numpy_array.get()); }
       const npy_intp *dims() const 
       { return PyArray_DIMS(m_numpy_array.get()); }
+      const npy_intp dim(npy_intp i) const 
+      { return PyArray_DIM(m_numpy_array.get(), i); }
       const npy_intp *strides() const 
       { return PyArray_STRIDES(m_numpy_array.get()); }
-      npy_intp min_stride() const
-      {
-        if (ndim() == 0)
-          return itemsize();
-        else
-          return *std::min_element(strides(), strides()+ndim());
-      }
+      const npy_intp stride(npy_intp i) const 
+      { return PyArray_STRIDE(m_numpy_array.get(), i); }
+
       npy_intp itemsize() const
       { return sizeof(T); }
       bool writable() const
@@ -247,13 +268,13 @@ namespace pyublas
       const_reference operator [] (size_type i) const 
       {
         BOOST_UBLAS_CHECK(i < size(), boost::numeric::ublas::bad_index());
-        return data()[i];
+        return begin()[i];
       }
 
       reference operator [] (size_type i) 
       {
         BOOST_UBLAS_CHECK(i < size(), boost::numeric::ublas::bad_index());
-        return data()[i];
+        return begin()[i];
       }
 
       // Assignment
@@ -285,26 +306,65 @@ namespace pyublas
 
       typedef const_pointer const_iterator;
 
-      const_iterator begin () const 
+    protected:
+
+      npy_intp max_pos_stride_index() const
       {
-        return data();
+        npy_intp current_idx = -1;
+        npy_intp current_max = 0;
+        for (unsigned i = 0; i < ndim(); ++i)
+        {
+          npy_intp si = stride(i);
+          if (si > current_max)
+          {
+            current_max = si;
+            current_idx = i;
+          }
+        }
+
+        return current_idx;
       }
 
-      const_iterator end () const 
+    public:
+      const_iterator begin() const 
       {
-        return data() + size();
+        const_iterator result = data();
+        for (unsigned i = 0; i < ndim(); ++i)
+        {
+          const npy_intp si = stride(i)/npy_intp(sizeof(T));
+          const npy_intp di = dim(i);
+          if (si < 0 && di)
+            result += si*(di-1);
+        }
+
+        return result;
+      }
+
+      const_iterator end() const 
+      { 
+        const npy_intp mpsi = max_pos_stride_index();
+
+        if (mpsi != -1)
+        {
+          const npy_intp mps = stride(mpsi)/npy_intp(sizeof(T));
+          return data() + mps*dim(mpsi);
+        }
+        else
+          return data()+1;
       }
 
       typedef pointer iterator;
 
-      iterator begin () 
+      iterator begin() 
       {
-        return data();
+        return const_cast<iterator>(
+            const_cast<numpy_array const *>(this)->begin());
       }
 
-      iterator end () 
-      {
-        return data() + size();
+      iterator end() 
+      { 
+        return const_cast<iterator>(
+            const_cast<numpy_array const *>(this)->end());
       }
 
       // Reverse iterators
@@ -439,10 +499,12 @@ namespace pyublas
       { return static_cast<const Derived *>(this)->array().ndim(); }
       const npy_intp *dims() const 
       { return static_cast<const Derived *>(this)->array().dims(); }
+      const npy_intp dim(npy_intp i) const 
+      { return static_cast<const Derived *>(this)->array().dim(i); }
       const npy_intp *strides() const 
       { return static_cast<const Derived *>(this)->array().strides(); }
-      npy_intp min_stride() const
-      { return static_cast<const Derived *>(this)->array().min_stride(); }
+      const npy_intp stride(npy_intp i) const 
+      { return static_cast<const Derived *>(this)->array().stride(i); }
       npy_intp itemsize() const
       { return sizeof(typename Derived::value_type); }
       bool writable() const 
@@ -521,6 +583,12 @@ namespace pyublas
 
 
 
+  template <class T>
+  class numpy_strided_vector;
+
+
+
+
   template <class T>
   class numpy_vector
   : public boost::numeric::ublas::vector<T, numpy_array<T> >,
@@ -580,18 +648,39 @@ namespace pyublas
       const super &as_ublas() const
       { return *this; }
 
-      boost::numeric::ublas::vector_slice<numpy_vector>
-        as_strided()
+      boost::numeric::ublas::slice stride_slice() const
       {
-        npy_intp ms = func::min_stride()/sizeof(T);
-        return subslice(*this, 0, ms, this->size()/ms);
+        if (this->ndim() == 1)
+        {
+          npy_intp stride = this->stride(0)/npy_intp(sizeof(T));
+          typename super::size_type sz = this->size();
+
+          if (stride < 0)
+          {
+            return boost::numeric::ublas::slice(
+                sz-1, stride, this->dim(0));
+          }
+          else
+            return boost::numeric::ublas::slice(0, stride, this->dim(0));
+        }
+        else if (this->ndim() == 0)
+          return boost::numeric::ublas::slice(0, 1, 1);
+        else
+          PYUBLAS_PYERROR(ValueError, 
+              "cannot generate a stride-respecting 1D slice "
+              "for 2D or higher arrays");
+      }
+
+      numpy_strided_vector<T> as_strided()
+      {
+        return numpy_strided_vector<T>(*this, stride_slice());
       }
 
       boost::numeric::ublas::vector_slice<const numpy_vector>
         as_strided() const
       {
-        npy_intp ms = func::min_stride()/sizeof(T);
-        return subslice(*this, 0, ms, this->size()/ms);
+        return boost::numeric::ublas::vector_slice<const numpy_vector>
+          (*this, stride_slice());
       }
 
       numpy_array<T> &array()
@@ -603,56 +692,63 @@ namespace pyublas
 
 
 
-#if 0
+  namespace detail
+  {
+    /* This is dumb, but necessary: In numpy_strided_vector, the 
+     * vector needs to be initialized before the slice referring 
+     * to it. But if the vector is a member, and the slice is a 
+     * base class, that won't happen. Therefore, move the vector
+     * to an artificial base class that can be specified first in
+     * the base-specifier-list. (cf. 12.6.2.5 in '96 working paper)
+     */
+    template <class V>
+    class vector_holder
+    {
+      public:
+        V m_vector;
+
+        vector_holder(const V &v)
+          : m_vector(v)
+        { }
+    };
+  }
+
+
+
+
   template <class T>
   class numpy_strided_vector
-  : public boost::numeric::ublas::vector_slice< numpy_vector<T> >,
+  : public detail::vector_holder<numpy_vector<T> >,
+    public boost::numeric::ublas::vector_slice< numpy_vector<T> >,
   public detail::vector_functionality<numpy_strided_vector<T>,
     boost::numeric::ublas::vector_slice< numpy_vector<T> >
    >
   {
     private:
+      typedef 
+        detail::vector_holder<numpy_vector<T> >
+        vector_holder;
       typedef 
         boost::numeric::ublas::vector_slice< numpy_vector<T> >
         super;
-      typename super::vector_type m_vector;
 
     public:
-      numpy_strided_vector ()
-        : super(m_vector)
-      {}
-
       // observe that PyObject handles are implicitly convertible
       // to numpy_array
       numpy_strided_vector(const numpy_array<T> &s)
-        : super(s.size(), s)
+        : vector_holder(s),
+        super(this->m_vector, boost::numeric::ublas::slice(0, 1, s.size()))
       {
       }
 
-      numpy_strided_vector(int ndim_, const npy_intp *dims_)
-        : super(size_from_dims(ndim_, dims_), 
-            numpy_array<T>(ndim_, dims_))
+      numpy_strided_vector(const numpy_strided_vector &v)
+        : vector_holder(v.m_vector),
+        super(this->m_vector, boost::numeric::ublas::slice(
+              v.start(), v.stride(), v.size()))
       { }
 
-      explicit 
-        numpy_strided_vector(typename super::size_type size)
-        : super(size) 
-        { }
-
-      numpy_strided_vector (
-          typename super::size_type size, 
-          const typename super::value_type &init)
-        : super(size, init) 
-      {}
-
-
-      numpy_strided_vector (const numpy_strided_vector &v)
-        : super(v)
-      { }
-
-      template<class AE>
-      numpy_strided_vector(const boost::numeric::ublas::vector_expression<AE> &ae)
-      : super(ae)
+      numpy_strided_vector(numpy_vector<T> &v, boost::numeric::ublas::slice const &s)
+        : vector_holder(v), super(this->m_vector, s)
       { }
 
       // as-ublas accessor
@@ -661,16 +757,19 @@ namespace pyublas
 
       const super &as_ublas() const
       { return *this; }
-      /*
-      boost::numeric::ublas::vector_slice<numpy_vector>
-        as_strided()
-      {
-        npy_intp ms = min_stride()/sizeof(T);
-        return subslice(*this, 0, ms, this->size()/ms);
-      }
-      */
+
+      numpy_array<T> &array()
+      { return this->m_vector.data(); }
+
+      numpy_array<T> const &array() const
+      { return this->m_vector.data(); }
+
+      typename super::size_type stride() const
+      { return super::stride(); }
+
+      npy_intp stride(npy_intp i) const
+      { return this->m_vector.stride(i); }
   };
-#endif
 
 
 
@@ -744,6 +843,44 @@ namespace pyublas
 
 
 
+  // conversion tags ----------------------------------------------------------
+  template <class Contained>
+  class invalid_ok
+  {
+    private:
+      Contained m_contained;
+
+    public:
+      typedef Contained contained_type;
+
+      invalid_ok(Contained c)
+        : m_contained(c)
+      { }
+
+      const Contained &operator*() const
+      {
+        return m_contained;
+      }
+
+      Contained &operator*()
+      {
+        return m_contained;
+      }
+
+      const Contained *operator->() const
+      {
+        return &m_contained;
+      }
+
+      Contained *operator->()
+      {
+        return &m_contained;
+      }
+  };
+
+
+
+
   // data member treatment ----------------------------------------------------
   template <class T, class C>
   class by_value_rw_member_visitor 
@@ -843,5 +980,4 @@ namespace boost { namespace numeric { namespace bindings { namespace traits {
 
 
 
-
 #endif
