@@ -50,6 +50,19 @@ namespace
       data->convertible = storage;
     }
 
+    static void construct_invalid_ok(
+        PyObject* obj, 
+        py::converter::rvalue_from_python_stage1_data* data)
+    {
+      typedef invalid_ok<target_type> inv_ok;
+      void* storage = ((py::converter::rvalue_from_python_storage<inv_ok>*)data)->storage.bytes;
+
+      new (storage) inv_ok(target_type(py::handle<>(py::borrowed(obj))));
+
+      // record successful construction
+      data->convertible = storage;
+    }
+
     template <class RealTgtType>
     static void construct_indirect(
         PyObject* obj, 
@@ -92,20 +105,38 @@ namespace
       typedef array_converter_base<VectorType> super;
 
     public:
+      static void construct_strided(
+          PyObject* obj, 
+          py::converter::rvalue_from_python_stage1_data* data)
+      {
+        typedef numpy_strided_vector<typename super::value_type> strided_vec;
+        void* storage = ((py::converter::rvalue_from_python_storage<strided_vec>*)data)->storage.bytes;
+
+        typename super::target_type vec(py::handle<>(py::borrowed(obj)));
+        new (storage) strided_vec(vec, vec.stride_slice());
+
+        // record successful construction
+        data->convertible = storage;
+      }
+
+
       static void *check(PyObject* obj)
       {
         if (!PyArray_Check(obj))
           return 0;
         if (PyArray_TYPE(obj) != get_typenum(typename super::value_type()))
           return 0;
-        if (!PyArray_CHKFLAGS(obj, NPY_ALIGNED))
-          return 0;
-        if (PyArray_CHKFLAGS(obj, NPY_NOTSWAPPED))
-          return 0;
 
         return obj;
       }
 
+      static void *check_invalid_ok(PyObject* obj)
+      {
+        if (obj == Py_None)
+          return obj;
+        return check(obj);
+      }
+
       // needs to be overridden to copy strided version
       template <class RealTgtType>
       static void construct_indirect(
@@ -142,8 +173,6 @@ namespace
           return 0;
         if (PyArray_TYPE(obj) != get_typenum(typename super::value_type()))
           return 0;
-        if (!PyArray_CHKFLAGS(obj, NPY_ALIGNED))
-          return 0;
         if (PyArray_NDIM(obj) != 2)
           return 0;
 
@@ -171,6 +200,12 @@ namespace
         return obj;
       }
 
+      static void *check_invalid_ok(PyObject* obj)
+      {
+        if (obj == Py_None)
+          return obj;
+        return check(obj);
+      }
   };
 
 
@@ -191,6 +226,15 @@ namespace
 #endif
         );
 
+    py::converter::registry::push_back(
+        &Converter::check_invalid_ok
+        , &Converter::construct_invalid_ok
+        , py::type_id<invalid_ok<typename Converter::target_type> >()
+#ifndef BOOST_PYTHON_NO_PY_SIGNATURES
+        , &get_PyArray_Type
+#endif
+        );
+
     py::to_python_converter<
       typename Converter::target_type, typename Converter::to_python>();
   }
@@ -198,6 +242,24 @@ namespace
 
 
 
+  template <class Converter>
+  void register_vector_converter()
+  {
+    py::converter::registry::push_back(
+        &Converter::check
+        , &Converter::construct_strided
+        , py::type_id<numpy_strided_vector<typename Converter::value_type> >()
+#ifndef BOOST_PYTHON_NO_PY_SIGNATURES
+        , &get_PyArray_Type
+#endif
+        );
+
+    register_array_converter<Converter>();
+  }
+
+
+
+
   template <class Converter, class RealTgtType>
   void register_indirect_array_converter()
   {
@@ -272,7 +334,7 @@ namespace
     typedef matrix_converter<cm_mat> cm_mat_converter;
     typedef matrix_converter<rm_mat> rm_mat_converter;
 
-    register_array_converter<vec_converter>();
+    register_vector_converter<vec_converter>();
     register_array_converter<cm_mat_converter>();
     register_array_converter<rm_mat_converter>();
 
